/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
package org.ow2.petals.se.validation.su;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.List;

import javax.xml.XMLConstants;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.SchemaFactory;

import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.se.validation.Constants;
import org.ow2.petals.se.validation.ValidationComponent;
import org.ow2.petals.se.validation.util.XsdResourceResolver;
import org.xml.sax.SAXException;

/**
 * @author rnaudin
 * 
 */
public class SUManager extends AbstractServiceUnitManager {

    /**
     * Default constructor.
     * 
     * @param component
     */
    public SUManager(AbstractComponent component) {
        super(component);
    }

    @Override
    protected void doDeploy(String serviceUnitName, String suRootPath, Jbi jbiDescriptor)
            throws PEtALSCDKException {
        // control that there is only one Provides section in the SU
        if (jbiDescriptor.getServices().getConsumes().size() != 0) {
            throw new PEtALSCDKException("Consumes section are not supported by this component");
        }

        final List<Provides> providesList = jbiDescriptor.getServices().getProvides();
        if (providesList == null || providesList.size() != 1) {
            throw new PEtALSCDKException("Only one Provides section supported");
        }

        // control that there is the required parameter, as there is for the
        // moment no validation against component level XSD
        final ServiceUnitDataHandler suDatahandler = getSUDataHandlerForService(providesList.get(0));
        final ConfigurationExtensions extensions = suDatahandler
                .getConfigurationExtensions(providesList.get(0));
        if (extensions.size() == 0 || extensions.size() > 1) {
            throw new PEtALSCDKException(
                    "Provides section must contains only the Validation parameter '"
                            + Constants.XSD_PATH + "'");
        }
        final String schemaFileName = extensions.get(Constants.XSD_PATH);
        if (StringHelper.isNullOrEmpty(schemaFileName)) {
            throw new PEtALSCDKException("Provides section must contains a Validation parameter '"
                    + Constants.XSD_PATH + "'");
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.su.AbstractServiceUnitManager#doInit
     * (java.lang.String, java.lang.String)
     */
    @Override
    protected void doInit(final String serviceUnitName, final String suRootPath)
            throws PEtALSCDKException {
        // Load the schema
        final Jbi jbiDescriptor = getServiceUnitDataHandlers().get(serviceUnitName).getDescriptor();
        final Provides provides = jbiDescriptor.getServices().getProvides().get(0);
        final ServiceUnitDataHandler suDatahandler = getSUDataHandlerForService(provides);
        final ConfigurationExtensions extensions = suDatahandler
                .getConfigurationExtensions(provides);
        final String schemaFileName = extensions.get(Constants.XSD_PATH);
        try {
            this.addSchema(schemaFileName, suRootPath, provides.getEndpointName());
        } catch (SAXException e) {
            throw new PEtALSCDKException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.su.AbstractServiceUnitManager#doShutdown
     * (java.lang.String)
     */
    @Override
    protected void doShutdown(String serviceUnitName) throws PEtALSCDKException {
        final Jbi jbiDescriptor = getServiceUnitDataHandlers().get(serviceUnitName).getDescriptor();
        final String endpointName = jbiDescriptor.getServices().getProvides().get(0)
                .getEndpointName();
        ((ValidationComponent) this.component).getSchemas().remove(endpointName);
    }

    /**
     * Add the schema to the component Map.
     * 
     * @return
     * @throws SAXException
     */
    private void addSchema(final String schemaFileName, final String suRootPath,
            final String endpointName) throws SAXException, PEtALSCDKException {

        InputStream xsdInputStream;
        // try to get the XSD from the filesystem
        try {
            xsdInputStream = new FileInputStream(new File(suRootPath, schemaFileName));
        } catch (final FileNotFoundException e) {
            throw new PEtALSCDKException("Failed to retrieve XSD resource", e);
        }

        final Source source = new StreamSource(xsdInputStream);

        final SchemaFactory schemaFactory = SchemaFactory
                .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        final XsdResourceResolver resolver = new XsdResourceResolver(suRootPath, this.logger);
        schemaFactory.setResourceResolver(resolver);

        ((ValidationComponent) this.component).getSchemas().put(endpointName,
                schemaFactory.newSchema(source));
    }
}
